function [R,Cx,Cy,varargout]=Courbure(x,y,run)

%This function computes a finite-difference approximation of the local
%radius of curvature R[i] and of the center of curvature C[i] in Cartesian
%coordinates. The input are the column vectors containing the coordinates,
%both of length N, (x[i],y(x[i])) of a continuous curve L. The outputs are
%column vectors containing R, local radius of curvature of L, Cx and Cy,
%coordinates (Ox,Oy) of the local center of curvature, horizontal, list of
%points with horizontal tangents (dy=0 for three consecutive points),
%vertical, list of points with vertical tangents (dx=0 for 3 consecutive
%points), dxzero list of points such that dx=0. The first and last values
%in a column are set as NaN due to the centered finite-differencing.
%
%First three output arguments are compulsory, last three are optional.
%
% Use: [R,Cx,Cy,horizontal,vertical,dxzero]=Courbure(x,y)
%
% (c) Aymeric-Pierre B. Peyret 2010-2011

if nargin<3
    run=1;
end

sx=size(x);
sy=size(y);

if sx(1)~=sy(1)
    if sx(1)~=sy(2)
        error('Sizes...');
    elseif sy(1)~=sx(2)
        error('Sizes...');
    else
        y=y';
        sy=size(y);
    end
else
    if sx(2)~=sy(2)
        error('Sizes...');
    end
end

lx=length(x);
ly=length(y);


dydx=diff(y)./diff(x);

ld1=length(dydx);
dydx=.5*dydx(1:ld1-1)+.5*dydx(2:ld1);
dxdx=ones(size(dydx));
dxav=(diff(x(2:lx))+diff(x(1:lx-1)))/2;

d2ydx2=(diff(y(2:ly))./diff(x(2:lx)) -diff(y(1:ly-1))./diff(x(1:lx-1)))./dxav;
d2xdx2=zeros(size(d2ydx2));

R=((dxdx.^2+dydx.^2).^(3/2))./(dxdx.*d2ydx2-dydx.*d2xdx2);
% dyzero=find(dy==0);
% doubledyzero=find(diff(dyzero)==1);
% horizontal=1+dyzero(doubledyzero);


% dxzero=find(dx==0);
% doubledxzero=find(diff(dxzero)==1);
% vertical=1+dxzero(doubledxzero);

normnormal=sqrt(dydx.^2+dxdx.^2);
MCx=-R.*dydx./normnormal;
MCy=R./normnormal;
Cx=x(2:lx-1)+MCx;
Cy=y(2:ly-1)+MCy;

if run<2%runs the code on (x,y) rotated by +pi/4 instead of (x,y) to work around *some* dx.dy=0 issues
    x2=x*cos(pi/4)-y*sin(pi/4);
    y2=x*sin(pi/4)+y*cos(pi/4);
    [R1,Cx2,Cy2]=Courbure(x2,y2,2);
%     size(R1)
    Cx1=Cx2*cos(pi/4)+Cy2*sin(pi/4);
    Cy1=-Cx2*sin(pi/4)+Cy2*cos(pi/4);
    R1=R1(2:end-1);
    Cx1=Cx1(2:end-1);
    Cy1=Cy1(2:end-1);
    dx=diff(x);dy=diff(y);
    dx2=diff(x2);dy2=diff(y2);
%     size(R)
%     size(R1)
    dvar=[dx(find(abs(1-abs(R)./abs(R1))>5*10^(-5)))'/mean(dx);
          dy(find(abs(1-abs(R)./abs(R1))>5*10^(-5)))'/mean(dy);
        dx2(find(abs(1-abs(R)./abs(R1))>5*10^(-5)))'/mean(dx2);
        dy2(find(abs(1-abs(R)./abs(R1))>5*10^(-5)))'/mean(dy2)];
    dvar=(dvar-ones(4,1)*min(dvar))./(ones(4,1)*(max(dvar)-min(dvar)));
    dvar=[dvar(1,:).*dvar(2,:);dvar(3,:).*dvar(4,:)];
    [indexldvar,indexcdvar]=find(dvar'==0);
    keep2=indexldvar(find(indexcdvar==1))';
%     keep1=indexldvar(find(indexcdvar==2))';
    R(keep2)=abs(R1(keep2)).*sign(R(keep2));
    Cx(keep2)=abs(Cx1(keep2));
    Cy(keep2)=abs(Cy1(keep2));
end
% 
% if ~isempty(dxzero)&&(run<2)%runs the code on (y,x) instead of (x,y) to work around *some* dx=0 issues
%     x2=x*cos(pi/4)-y*sin(pi/4);
%     y2=x*sin(pi/4)+y*cos(pi/4);
%     
%     [R1,Cx1,Cy1]=Courbure(x2,y2,2);
%     
%     Cx1=Cx1*cos(pi/4)+Cy1*sin(pi/4);
%     Cy1=-Cx1*sin(pi/2)+Cy1*cos(pi/4);
%     
%     R1=-R1(2:end-1);
%     Cy1=Cy1(2:end-1);
%     Cx1=Cx1(2:end-1);
%     R(dxzero-1)=-R1(dxzero-1);
%     Cx(dxzero-1)=Cx1(dxzero-1);
%     Cy(dxzero-1)=Cy1(dxzero-1);
%     R(dxzero)=-R1(dxzero);
%     Cx(dxzero)=Cx1(dxzero);
%     Cy(dxzero)=Cy1(dxzero);
% end



% dxzero=1+dxzero;

R=[NaN;R;NaN];
Cx=[NaN;Cx;NaN];
Cy=[NaN;Cy;NaN];

nout = max(nargout,1)-1;
if (run<2)&&(nout>0)
    varargout(1)={keep2};
end

% 
% if nout>0
%     varargout(1)={horizontal};
% end
% if nout>1
%     varargout(2)={vertical};
% end
% if nout>2
%     varargout(3)={dxzero};
% end
